@@ -1,5 +1,5 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2009 Brazil
+/* Copyright(C) 2009-2011 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -192,7 +192,7 @@ struct _grn_com_header {
 
 GRN_API grn_com *grn_com_copen(grn_ctx *ctx, grn_com_event *ev, const char *dest, int port);
 GRN_API grn_rc grn_com_sopen(grn_ctx *ctx, grn_com_event *ev,
-                             const char *listen_address, int port,
+                             const char *bind_address, int port,
                              grn_msg_handler *func, struct hostent *he);
 
 GRN_API void grn_com_close_(grn_ctx *ctx, grn_com *com);
