@@ -144,6 +144,9 @@ GRN_API void grn_ctx_impl_err(grn_ctx *ctx);
   if (!(expr)) { QLERR("syntax error"); }\
 } while (0)
 
+#ifdef ERR
+#  undef ERR
+#endif /* ERR */
 #define ERR(rc,...) ERRSET(ctx, GRN_ERROR, (rc),  __VA_ARGS__)
 #define WARN(rc,...) ERRSET(ctx, GRN_WARN, (rc),  __VA_ARGS__)
 #define MERR(...) ERRSET(ctx, GRN_ALERT, GRN_NO_MEMORY_AVAILABLE,  __VA_ARGS__)
