@@ -39,14 +39,10 @@ typedef struct {
   grn_obj bottom_right_point_buffer;
   grn_geo_point *top_left;
   grn_geo_point *bottom_right;
-  grn_geo_point base;
   grn_geo_point min;
   grn_geo_point max;
-  int start;
-  int end;
-  int distance;
-  int diff_bit;
-  grn_geo_mesh_direction direction;
+  int rectangle_common_bit;
+  uint8_t rectangle_common_key[sizeof(grn_geo_point)];
 } in_rectangle_data;
 
 static int
@@ -70,36 +66,53 @@ compute_diff_bit(uint8_t *geo_key1, uint8_t *geo_key2)
   return i * 8 + diff_bit;
 }
 
+static void
+compute_min_and_max_key(uint8_t *key_base, int diff_bit,
+                        uint8_t *key_min, uint8_t *key_max)
+{
+  int diff_byte, diff_bit_mask;
+
+  diff_byte = diff_bit / 8;
+  diff_bit_mask = 0xff >> (diff_bit % 8);
+
+  if (diff_byte == sizeof(grn_geo_point)) {
+    if (key_min) { memcpy(key_min, key_base, diff_byte); }
+    if (key_max) { memcpy(key_max, key_base, diff_byte); }
+  } else {
+    if (key_min) {
+      memcpy(key_min, key_base, diff_byte + 1);
+      key_min[diff_byte] &= ~diff_bit_mask;
+      memset(key_min + diff_byte + 1, 0,
+             sizeof(grn_geo_point) - diff_byte - 1);
+    }
+
+    if (key_max) {
+      memcpy(key_max, key_base, diff_byte + 1);
+      key_max[diff_byte] |= diff_bit_mask;
+      memset(key_max + diff_byte + 1, 0xff,
+             sizeof(grn_geo_point) - diff_byte - 1);
+    }
+  }
+}
+
 static void
 compute_min_and_max(grn_geo_point *base_point, int diff_bit,
                     grn_geo_point *geo_min, grn_geo_point *geo_max)
 {
-  int diff_byte, diff_bit_mask;
   uint8_t geo_key_base[sizeof(grn_geo_point)];
   uint8_t geo_key_min[sizeof(grn_geo_point)];
   uint8_t geo_key_max[sizeof(grn_geo_point)];
 
-  diff_byte = diff_bit / 8;
-  diff_bit_mask = 0xff >> (diff_bit % 8);
   grn_gton(geo_key_base, base_point, sizeof(grn_geo_point));
-
-  if (diff_byte == sizeof(grn_geo_point)) {
-    memcpy(geo_key_min, geo_key_base, diff_byte);
-    memcpy(geo_key_max, geo_key_base, diff_byte);
-  } else {
-    memcpy(geo_key_min, geo_key_base, diff_byte + 1);
-    geo_key_min[diff_byte] &= ~diff_bit_mask;
-    memset(geo_key_min + diff_byte + 1, 0,
-           sizeof(grn_geo_point) - diff_byte - 1);
-
-    memcpy(geo_key_max, geo_key_base, diff_byte + 1);
-    geo_key_max[diff_byte] |= diff_bit_mask;
-    memset(geo_key_max + diff_byte + 1, 0xff,
-           sizeof(grn_geo_point) - diff_byte - 1);
+  compute_min_and_max_key(geo_key_base, diff_bit,
+                          geo_min ? geo_key_min : NULL,
+                          geo_max ? geo_key_max : NULL);
+  if (geo_min) {
+    grn_ntog((uint8_t *)geo_min, geo_key_min, sizeof(grn_geo_point));
+  }
+  if (geo_max) {
+    grn_ntog((uint8_t *)geo_max, geo_key_max, sizeof(grn_geo_point));
   }
-
-  grn_ntog((uint8_t *)geo_min, geo_key_min, sizeof(grn_geo_point));
-  grn_ntog((uint8_t *)geo_max, geo_key_max, sizeof(grn_geo_point));
 }
 
 /* #define GEO_DEBUG */
@@ -139,10 +152,91 @@ inspect_tid(grn_ctx *ctx, grn_id tid, grn_geo_point *point, double d)
   printf("tid: %d:%g", tid, d);
   grn_p_geo_point(ctx, point);
 }
+
+static void
+inspect_key(grn_ctx *ctx, uint8_t *key)
+{
+  int i;
+  for (i = 0; i < 8; i++) {
+    int j;
+    for (j = 0; j < 8; j++) {
+      printf("%d", (key[i] & (1 << (7 - j))) >> (7 - j));
+    }
+    printf(" ");
+  }
+  printf("\n");
+}
+
+static void
+print_key_mark(grn_ctx *ctx, int target_bit)
+{
+  int i;
+
+  for (i = 0; i < target_bit; i++) {
+    printf(" ");
+    if (i > 0 && i % 8 == 0) {
+      printf(" ");
+    }
+  }
+  if (i > 0 && i % 8 == 0) {
+    printf(" ");
+  }
+  printf("^\n");
+}
+
+static void
+inspect_cursor_entry(grn_ctx *ctx, grn_geo_cursor_entry *entry)
+{
+  grn_geo_point point;
+
+  printf("entry: ");
+  grn_ntog((uint8_t *)&point, entry->key, sizeof(grn_geo_point));
+  grn_p_geo_point(ctx, &point);
+  inspect_key(ctx, entry->key);
+  print_key_mark(ctx, entry->target_bit);
+
+  printf("     target bit:    %d\n", entry->target_bit);
+
+#define INSPECT_STATUS_FLAG(name) \
+  (entry->status_flags & GRN_GEO_CURSOR_ENTRY_STATUS_ ## name) ? "true" : "false"
+
+  printf("   top included:    %s\n", INSPECT_STATUS_FLAG(TOP_INCLUDED));
+  printf("bottom included:    %s\n", INSPECT_STATUS_FLAG(BOTTOM_INCLUDED));
+  printf("  left included:    %s\n", INSPECT_STATUS_FLAG(LEFT_INCLUDED));
+  printf(" right included:    %s\n", INSPECT_STATUS_FLAG(RIGHT_INCLUDED));
+  printf(" latitude inner:    %s\n", INSPECT_STATUS_FLAG(LATITUDE_INNER));
+  printf("longitude inner:    %s\n", INSPECT_STATUS_FLAG(LONGITUDE_INNER));
+
+#undef INSPECT_STATUS_FLAG
+}
+
+static void
+inspect_cursor_entry_targets(grn_ctx *ctx, grn_geo_cursor_entry *entry,
+                             uint8_t *top_left_key, uint8_t *bottom_right_key,
+                             grn_geo_cursor_entry *next_entry0,
+                             grn_geo_cursor_entry *next_entry1)
+{
+  printf("entry:        ");
+  inspect_key(ctx, entry->key);
+  printf("top-left:     ");
+  inspect_key(ctx, top_left_key);
+  printf("bottom-right: ");
+  inspect_key(ctx, bottom_right_key);
+  printf("next-entry-0: ");
+  inspect_key(ctx, next_entry0->key);
+  printf("next-entry-1: ");
+  inspect_key(ctx, next_entry1->key);
+  printf("              ");
+  print_key_mark(ctx, entry->target_bit + 1);
+}
 #else
 #  define inspect_mesh(...)
 #  define inspect_mesh_entry(...)
 #  define inspect_tid(...)
+#  define inspect_key(...)
+#  define print_key_mark(...)
+#  define inspect_cursor_entry(...)
+#  define inspect_cursor_entry_targets(...)
 #endif
 
 static int
@@ -842,11 +936,15 @@ in_rectangle_data_prepare(grn_ctx *ctx, grn_obj *index,
   }
 
   {
-    int distance, latitude_distance, longitude_distance;
+    int latitude_distance, longitude_distance;
+    int diff_bit;
+    grn_geo_point base;
     grn_geo_point *top_left, *bottom_right;
     grn_geo_point *geo_point_input;
     uint8_t geo_key_input[sizeof(grn_geo_point)];
     uint8_t geo_key_base[sizeof(grn_geo_point)];
+    uint8_t geo_key_top_left[sizeof(grn_geo_point)];
+    uint8_t geo_key_bottom_right[sizeof(grn_geo_point)];
 
     top_left = data->top_left;
     bottom_right = data->bottom_right;
@@ -854,40 +952,29 @@ in_rectangle_data_prepare(grn_ctx *ctx, grn_obj *index,
     latitude_distance = top_left->latitude - bottom_right->latitude;
     longitude_distance = bottom_right->longitude - top_left->longitude;
     if (latitude_distance > longitude_distance) {
-      data->direction = GRN_GEO_MESH_LATITUDE;
       geo_point_input = bottom_right;
-      data->base.latitude = bottom_right->latitude;
-      data->base.longitude = bottom_right->longitude - longitude_distance;
+      base.latitude = bottom_right->latitude;
+      base.longitude = bottom_right->longitude - longitude_distance;
     } else {
-      data->direction = GRN_GEO_MESH_LONGITUDE;
       geo_point_input = top_left;
-      data->base.latitude = top_left->latitude - latitude_distance;
-      data->base.longitude = top_left->longitude;
+      base.latitude = top_left->latitude - latitude_distance;
+      base.longitude = top_left->longitude;
     }
     grn_gton(geo_key_input, geo_point_input, sizeof(grn_geo_point));
-    grn_gton(geo_key_base, &(data->base), sizeof(grn_geo_point));
-    data->diff_bit = compute_diff_bit(geo_key_input, geo_key_base);
-    compute_min_and_max(&(data->base), data->diff_bit,
-                        &(data->min), &(data->max));
-    switch (data->direction) {
-    case GRN_GEO_MESH_LATITUDE :
-      data->distance = data->max.latitude - data->min.latitude + 1;
-      data->start = data->min.latitude;
-      data->end = top_left->latitude;
-      break;
-    case GRN_GEO_MESH_LONGITUDE :
-      data->distance = data->max.longitude - data->min.longitude + 1;
-      data->start = data->min.longitude;
-      data->end = bottom_right->longitude;
-      break;
-    }
+    grn_gton(geo_key_base, &base, sizeof(grn_geo_point));
+    diff_bit = compute_diff_bit(geo_key_input, geo_key_base);
+    compute_min_and_max(&base, diff_bit, &(data->min), &(data->max));
+
+    grn_gton(geo_key_top_left, top_left, sizeof(grn_geo_point));
+    grn_gton(geo_key_bottom_right, bottom_right, sizeof(grn_geo_point));
+    data->rectangle_common_bit =
+      compute_diff_bit(geo_key_top_left, geo_key_bottom_right) - 1;
+    compute_min_and_max_key(geo_key_top_left, data->rectangle_common_bit + 1,
+                            data->rectangle_common_key, NULL);
+
 #ifdef GEO_DEBUG
-    printf("direction: %s\n",
-           data->direction == GRN_GEO_MESH_LATITUDE ? "latitude" : "longitude");
     printf("base:         ");
-    grn_p_geo_point(ctx, &(data->base));
-    printf("input:        ");
-    grn_p_geo_point(ctx, geo_point_input);
+    grn_p_geo_point(ctx, &base);
     printf("min:          ");
     grn_p_geo_point(ctx, &(data->min));
     printf("max:          ");
@@ -896,20 +983,53 @@ in_rectangle_data_prepare(grn_ctx *ctx, grn_obj *index,
     grn_p_geo_point(ctx, top_left);
     printf("bottom-right: ");
     grn_p_geo_point(ctx, bottom_right);
-    printf("diff-bit:            %10d\n", data->diff_bit);
-    printf("start:               %10d\n", data->start);
-    printf("end:                 %10d\n", data->end);
-    printf("distance:            %10d\n", data->distance);
+    printf("rectangle-common-bit:%10d\n", data->rectangle_common_bit);
     printf("distance(latitude):  %10d\n", latitude_distance);
     printf("distance(longitude): %10d\n", longitude_distance);
 #endif
-    memcpy(&(data->base), &(data->min), sizeof(grn_geo_point));
   }
 
 exit :
   return ctx->rc;
 }
 
+#define SAME_BIT_P(a, b, n_bit)\
+  ((((uint8_t *)(a))[(n_bit) / 8] & (1 << (7 - ((n_bit) % 8)))) ==\
+   (((uint8_t *)(b))[(n_bit) / 8] & (1 << (7 - ((n_bit) % 8)))))
+
+#define CURSOR_ENTRY_UPDATE_STATUS(entry, name, other_key)\
+  if (SAME_BIT_P((entry)->key, (other_key), (entry)->target_bit)) {\
+    (entry)->status_flags |= GRN_GEO_CURSOR_ENTRY_STATUS_ ## name;\
+  } else {\
+    (entry)->status_flags &= ~GRN_GEO_CURSOR_ENTRY_STATUS_ ## name;\
+  }
+
+#define CURSOR_ENTRY_CHECK_STATUS(entry, name)\
+  ((entry)->status_flags & GRN_GEO_CURSOR_ENTRY_STATUS_ ## name)
+#define CURSOR_ENTRY_IS_INNER(entry)\
+  (((entry)->status_flags &\
+    (GRN_GEO_CURSOR_ENTRY_STATUS_LATITUDE_INNER |\
+     GRN_GEO_CURSOR_ENTRY_STATUS_LONGITUDE_INNER)) ==\
+   (GRN_GEO_CURSOR_ENTRY_STATUS_LATITUDE_INNER |\
+    GRN_GEO_CURSOR_ENTRY_STATUS_LONGITUDE_INNER))
+#define CURSOR_ENTRY_INCLUDED_IN_LATITUDE_DIRECTION(entry)\
+  ((entry)->status_flags &\
+   (GRN_GEO_CURSOR_ENTRY_STATUS_LATITUDE_INNER |\
+    GRN_GEO_CURSOR_ENTRY_STATUS_TOP_INCLUDED |\
+    GRN_GEO_CURSOR_ENTRY_STATUS_BOTTOM_INCLUDED))
+#define CURSOR_ENTRY_INCLUDED_IN_LONGITUDE_DIRECTION(entry)\
+  ((entry)->status_flags &\
+   (GRN_GEO_CURSOR_ENTRY_STATUS_LONGITUDE_INNER |\
+    GRN_GEO_CURSOR_ENTRY_STATUS_LEFT_INCLUDED |\
+    GRN_GEO_CURSOR_ENTRY_STATUS_RIGHT_INCLUDED))
+
+#define SET_N_BIT(a, n_bit)\
+  ((uint8_t *)(a))[((n_bit) / 8)] ^= (1 << (7 - ((n_bit) % 8)))
+
+#define N_BIT(a, n_bit)\
+  ((((uint8_t *)(a))[((n_bit) / 8)] &\
+    (1 << (7 - ((n_bit) % 8)))) >> (1 << (7 - ((n_bit) % 8))))
+
 grn_obj *
 grn_geo_cursor_open_in_rectangle(grn_ctx *ctx,
                                  grn_obj *index,
@@ -921,6 +1041,7 @@ grn_geo_cursor_open_in_rectangle(grn_ctx *ctx,
   grn_geo_cursor_in_rectangle *cursor = NULL;
   in_rectangle_data data;
 
+  GRN_API_ENTER;
   GRN_VOID_INIT(&(data.top_left_point_buffer));
   GRN_VOID_INIT(&(data.bottom_right_point_buffer));
   if (in_rectangle_data_prepare(ctx, index, top_left_point, bottom_right_point,
@@ -935,26 +1056,315 @@ grn_geo_cursor_open_in_rectangle(grn_ctx *ctx,
     goto exit;
   }
 
-  GRN_DB_OBJ_SET_TYPE(cursor, GRN_CURSOR_COLUMN_GEO_INDEX);
   cursor->pat = data.pat;
   cursor->index = index;
-  cursor->diff_bit = data.diff_bit;
-  cursor->start_mesh_point = data.start;
-  cursor->end_mesh_point = data.end;
-  cursor->distance = data.distance;
-  cursor->direction = data.direction;
   memcpy(&(cursor->top_left), data.top_left, sizeof(grn_geo_point));
   memcpy(&(cursor->bottom_right), data.bottom_right, sizeof(grn_geo_point));
-  memcpy(&(cursor->base), &(data.base), sizeof(grn_geo_point));
+  grn_gton(cursor->top_left_key, data.top_left, sizeof(grn_geo_point));
+  grn_gton(cursor->bottom_right_key, data.bottom_right, sizeof(grn_geo_point));
   cursor->pat_cursor = NULL;
   cursor->ii_cursor = NULL;
   cursor->offset = offset;
   cursor->rest = limit;
 
+  cursor->current_entry = 0;
+  {
+    grn_geo_cursor_entry *entry;
+
+    entry = &(cursor->entries[cursor->current_entry]);
+    entry->target_bit = data.rectangle_common_bit;
+    memcpy(entry->key, data.rectangle_common_key, sizeof(grn_geo_point));
+    entry->status_flags =
+      GRN_GEO_CURSOR_ENTRY_STATUS_TOP_INCLUDED |
+      GRN_GEO_CURSOR_ENTRY_STATUS_BOTTOM_INCLUDED |
+      GRN_GEO_CURSOR_ENTRY_STATUS_LEFT_INCLUDED |
+      GRN_GEO_CURSOR_ENTRY_STATUS_RIGHT_INCLUDED;
+    if (data.min.latitude == data.bottom_right->latitude &&
+        data.max.latitude == data.top_left->latitude) {
+      entry->status_flags |= GRN_GEO_CURSOR_ENTRY_STATUS_LATITUDE_INNER;
+    }
+    if (data.min.longitude == data.top_left->longitude &&
+        data.max.longitude == data.bottom_right->longitude) {
+      entry->status_flags |= GRN_GEO_CURSOR_ENTRY_STATUS_LONGITUDE_INNER;
+    }
+  }
+  {
+    const char *minimum_reduce_bit_env;
+    cursor->minimum_reduce_bit = 0;
+    minimum_reduce_bit_env = getenv("GRN_GEO_IN_RECTANGLE_MINIMUM_REDUCE_BIT");
+    if (minimum_reduce_bit_env) {
+      cursor->minimum_reduce_bit = atoi(minimum_reduce_bit_env);
+    }
+    if (cursor->minimum_reduce_bit < 1) {
+      cursor->minimum_reduce_bit = 1;
+    }
+  }
+  GRN_DB_OBJ_SET_TYPE(cursor, GRN_CURSOR_COLUMN_GEO_INDEX);
+  {
+    grn_obj *db;
+    grn_id id;
+    db = grn_ctx_db(ctx);
+    id = grn_obj_register(ctx, db, NULL, 0);
+    DB_OBJ(cursor)->header.domain = GRN_ID_NIL;
+    DB_OBJ(cursor)->range = GRN_ID_NIL;
+    grn_db_obj_init(ctx, db, id, DB_OBJ(cursor));
+  }
+
 exit :
   grn_obj_unlink(ctx, &(data.top_left_point_buffer));
   grn_obj_unlink(ctx, &(data.bottom_right_point_buffer));
-  return (grn_obj *)cursor;
+  GRN_API_RETURN((grn_obj *)cursor);
+}
+
+static inline grn_bool
+grn_geo_cursor_entry_next_push(grn_ctx *ctx,
+                               grn_geo_cursor_in_rectangle *cursor,
+                               grn_geo_cursor_entry *entry)
+{
+  grn_geo_cursor_entry *next_entry;
+  grn_geo_point entry_base;
+  grn_table_cursor *pat_cursor;
+  grn_bool pushed = GRN_FALSE;
+
+  grn_ntog((uint8_t*)(&entry_base), entry->key, sizeof(grn_geo_point));
+  pat_cursor = grn_table_cursor_open(ctx,
+                                     cursor->pat,
+                                     &entry_base,
+                                     entry->target_bit + 1,
+                                     NULL, 0,
+                                     0, -1,
+                                     GRN_CURSOR_PREFIX|GRN_CURSOR_SIZE_BY_BIT);
+  if (pat_cursor) {
+    if (grn_table_cursor_next(ctx, pat_cursor)) {
+      next_entry = &(cursor->entries[++cursor->current_entry]);
+      memcpy(next_entry, entry, sizeof(grn_geo_cursor_entry));
+      pushed = GRN_TRUE;
+    }
+    grn_table_cursor_close(ctx, pat_cursor);
+  }
+
+  return pushed;
+}
+
+static inline grn_bool
+grn_geo_cursor_entry_next(grn_ctx *ctx,
+                          grn_geo_cursor_in_rectangle *cursor,
+                          grn_geo_cursor_entry *entry)
+{
+  uint8_t *top_left_key = cursor->top_left_key;
+  uint8_t *bottom_right_key = cursor->bottom_right_key;
+  int max_target_bit = GRN_GEO_KEY_MAX_BITS - cursor->minimum_reduce_bit;
+
+  if (cursor->current_entry < 0) {
+    return GRN_FALSE;
+  }
+
+  memcpy(entry,
+         &(cursor->entries[cursor->current_entry--]),
+         sizeof(grn_geo_cursor_entry));
+  while (GRN_TRUE) {
+    grn_geo_cursor_entry next_entry0, next_entry1;
+    grn_bool pushed = GRN_FALSE;
+
+    /*
+      top_left_key: tl
+      bottom_right_key: br
+
+      e.g.: top_left_key is at the top left sub mesh and
+            bottom_right_key is at the bottom right sub mesh.
+            top_left_key is also at the top left - bottom right
+            sub-sub mesh and
+            bottom_right_key is at the bottom right - bottom left
+            sub-sub mesh.
+
+      ^latitude +----+----+----+----+
+      |       1 |1010|1011|1110|1111|
+      |         |    |    |    |    |
+      |    1    +----+----+----+----+
+     \/       0 |1000|1001|1100|1101|
+                |    | tl |    |    |
+                +----+----+----+----+
+              1 |0010|0011|0110|0111|
+                |    |    |    |    |
+           0    +----+----+----+----+
+              0 |0000|0001|0100|0101|
+                |    |    | br |    |
+                +----+----+----+----+
+                  0    1    0    1
+                 |-------| |-------|
+                     0         1
+                <------>
+                longitude
+
+      entry.target_bit + 1      -> next_entry0
+      entry.target_bit + 1 and entry.key ^ (entry.target_bit + 1) in bit
+                                -> next_entry1
+
+      entry: represents the biggest mesh.
+             (1010, 1011, 1110, 1111,
+              1000, 1001, 1100, 1101,
+              0010, 0011, 0110, 0111,
+              0000, 0001, 0100, 0101)
+      next_entry0: represents bottom sub-mesh.
+             (0010, 0011, 0110, 0111,
+              0000, 0001, 0100, 0101)
+      next_entry1: represents top sub-mesh.
+             (1010, 1011, 1110, 1111,
+              1000, 1001, 1100, 1101)
+
+      entry->status_flags       = TOP_INCLUDED |
+                                  BOTTOM_INCLUDED |
+                                  LEFT_INCLUDED |
+                                  RIGHT_INCLUDED
+      next_entry0->status_flags = BOTTOM_INCLUDED |
+                                  LEFT_INCLUDED |
+                                  RIGHT_INCLUDED
+      next_entry1->status_flags = TOP_INCLUDED |
+                                  LEFT_INCLUDED |
+                                  RIGHT_INCLUDED
+
+      Both next_entry1 and next_entry0 are pushed to the stack in cursor.
+    */
+
+#ifdef GEO_DEBUG
+    inspect_cursor_entry(ctx, entry);
+#endif
+
+    if (entry->target_bit >= max_target_bit) {
+#ifdef GEO_DEBUG
+      printf("%d: force stopping to reduce a mesh\n", entry->target_bit);
+#endif
+      break;
+    }
+
+    if (CURSOR_ENTRY_IS_INNER(entry)) {
+#ifdef GEO_DEBUG
+      printf("%d: inner entries\n", entry->target_bit);
+#endif
+      break;
+    }
+
+    memcpy(&next_entry0, entry, sizeof(grn_geo_cursor_entry));
+    next_entry0.target_bit++;
+    memcpy(&next_entry1, entry, sizeof(grn_geo_cursor_entry));
+    next_entry1.target_bit++;
+    SET_N_BIT(next_entry1.key, next_entry1.target_bit);
+
+#ifdef GEO_DEBUG
+    inspect_cursor_entry_targets(ctx, entry, top_left_key, bottom_right_key,
+                                 &next_entry0, &next_entry1);
+#endif
+
+    if ((entry->target_bit + 1) % 2 == 0) {
+      if (CURSOR_ENTRY_CHECK_STATUS(entry, TOP_INCLUDED)) {
+        CURSOR_ENTRY_UPDATE_STATUS(&next_entry0, TOP_INCLUDED, top_left_key);
+        CURSOR_ENTRY_UPDATE_STATUS(&next_entry1, TOP_INCLUDED, top_left_key);
+      }
+      if (CURSOR_ENTRY_CHECK_STATUS(entry, BOTTOM_INCLUDED)) {
+        CURSOR_ENTRY_UPDATE_STATUS(&next_entry0, BOTTOM_INCLUDED,
+                                   bottom_right_key);
+        CURSOR_ENTRY_UPDATE_STATUS(&next_entry1, BOTTOM_INCLUDED,
+                                   bottom_right_key);
+      }
+
+      if (CURSOR_ENTRY_CHECK_STATUS(entry, TOP_INCLUDED) &&
+          !CURSOR_ENTRY_CHECK_STATUS(entry, BOTTOM_INCLUDED) &&
+          CURSOR_ENTRY_CHECK_STATUS(&next_entry1, TOP_INCLUDED)) {
+        next_entry0.status_flags |= GRN_GEO_CURSOR_ENTRY_STATUS_LATITUDE_INNER;
+      } else if (!CURSOR_ENTRY_CHECK_STATUS(entry, TOP_INCLUDED) &&
+                 CURSOR_ENTRY_CHECK_STATUS(entry, BOTTOM_INCLUDED) &&
+                 CURSOR_ENTRY_CHECK_STATUS(&next_entry0, BOTTOM_INCLUDED)) {
+        next_entry1.status_flags |= GRN_GEO_CURSOR_ENTRY_STATUS_LATITUDE_INNER;
+      }
+
+      if (CURSOR_ENTRY_INCLUDED_IN_LATITUDE_DIRECTION(&next_entry1)) {
+        if (grn_geo_cursor_entry_next_push(ctx, cursor, &next_entry1)) {
+          pushed = GRN_TRUE;
+#ifdef GEO_DEBUG
+          printf("%d: latitude: push 1\n", next_entry1.target_bit);
+#endif
+        }
+      }
+      if (CURSOR_ENTRY_INCLUDED_IN_LATITUDE_DIRECTION(&next_entry0)) {
+        if (grn_geo_cursor_entry_next_push(ctx, cursor, &next_entry0)) {
+          pushed = GRN_TRUE;
+#ifdef GEO_DEBUG
+          printf("%d: latitude: push 0\n", next_entry0.target_bit);
+#endif
+        }
+      }
+    } else {
+      if (CURSOR_ENTRY_CHECK_STATUS(entry, RIGHT_INCLUDED)) {
+        CURSOR_ENTRY_UPDATE_STATUS(&next_entry0, RIGHT_INCLUDED,
+                                   bottom_right_key);
+        CURSOR_ENTRY_UPDATE_STATUS(&next_entry1, RIGHT_INCLUDED,
+                                   bottom_right_key);
+      }
+      if (CURSOR_ENTRY_CHECK_STATUS(entry, LEFT_INCLUDED)) {
+        CURSOR_ENTRY_UPDATE_STATUS(&next_entry0, LEFT_INCLUDED, top_left_key);
+        CURSOR_ENTRY_UPDATE_STATUS(&next_entry1, LEFT_INCLUDED, top_left_key);
+      }
+
+      if (CURSOR_ENTRY_CHECK_STATUS(entry, LEFT_INCLUDED) &&
+          !CURSOR_ENTRY_CHECK_STATUS(entry, RIGHT_INCLUDED) &&
+          CURSOR_ENTRY_CHECK_STATUS(&next_entry0, LEFT_INCLUDED)) {
+        next_entry1.status_flags |= GRN_GEO_CURSOR_ENTRY_STATUS_LONGITUDE_INNER;
+      } else if (!CURSOR_ENTRY_CHECK_STATUS(entry, LEFT_INCLUDED) &&
+                 CURSOR_ENTRY_CHECK_STATUS(entry, RIGHT_INCLUDED) &&
+                 CURSOR_ENTRY_CHECK_STATUS(&next_entry1, RIGHT_INCLUDED)) {
+        next_entry0.status_flags |= GRN_GEO_CURSOR_ENTRY_STATUS_LONGITUDE_INNER;
+      }
+
+      if (CURSOR_ENTRY_INCLUDED_IN_LONGITUDE_DIRECTION(&next_entry1)) {
+        if (grn_geo_cursor_entry_next_push(ctx, cursor, &next_entry1)) {
+          pushed = GRN_TRUE;
+#ifdef GEO_DEBUG
+          printf("%d: longitude: push 1\n", next_entry1.target_bit);
+#endif
+        }
+      }
+      if (CURSOR_ENTRY_INCLUDED_IN_LONGITUDE_DIRECTION(&next_entry0)) {
+        if (grn_geo_cursor_entry_next_push(ctx, cursor, &next_entry0)) {
+          pushed = GRN_TRUE;
+#ifdef GEO_DEBUG
+          printf("%d: longitude: push 0\n", next_entry0.target_bit);
+#endif
+        }
+      }
+    }
+
+    if (pushed) {
+#ifdef GEO_DEBUG
+      int i;
+
+      printf("%d: pushed\n", entry->target_bit);
+      printf("stack:\n");
+      for (i = cursor->current_entry; i >= 0; i--) {
+        grn_geo_cursor_entry *stack_entry;
+        stack_entry = &(cursor->entries[i]);
+        printf("%2d: ", i);
+        inspect_key(ctx, stack_entry->key);
+        printf("    ");
+        print_key_mark(ctx, stack_entry->target_bit);
+      }
+#endif
+      memcpy(entry,
+             &(cursor->entries[cursor->current_entry--]),
+             sizeof(grn_geo_cursor_entry));
+#ifdef GEO_DEBUG
+      printf("%d: pop entry\n", entry->target_bit);
+#endif
+    } else {
+      break;
+    }
+  }
+
+#ifdef GEO_DEBUG
+  printf("found:\n");
+  inspect_cursor_entry(ctx, entry);
+#endif
+
+  return GRN_TRUE;
 }
 
 typedef grn_bool (*grn_geo_cursor_callback)(grn_ctx *ctx, grn_ii_posting *posting, void *user_data);
@@ -969,11 +1379,7 @@ grn_geo_cursor_each(grn_ctx *ctx, grn_obj *geo_cursor,
   grn_ii *ii;
   grn_ii_cursor *ii_cursor;
   grn_ii_posting *posting = NULL;
-  grn_geo_point *current, *base, *top_left, *bottom_right;
-  int diff_bit, distance, end_mesh_point;
-  grn_geo_mesh_direction direction;
-  int i = 0;
-  int mesh_point;
+  grn_geo_point *current, *top_left, *bottom_right;
   grn_id index_id;
 
   cursor = (grn_geo_cursor_in_rectangle *)geo_cursor;
@@ -986,21 +1392,23 @@ grn_geo_cursor_each(grn_ctx *ctx, grn_obj *geo_cursor,
   ii = (grn_ii *)(cursor->index);
   ii_cursor = cursor->ii_cursor;
   current = &(cursor->current);
-  base = &(cursor->base);
   top_left = &(cursor->top_left);
   bottom_right = &(cursor->bottom_right);
-  diff_bit = cursor->diff_bit;
-  distance = cursor->distance;
-  end_mesh_point = cursor->end_mesh_point;
-  direction = cursor->direction;
 
   while (GRN_TRUE) {
     if (!pat_cursor) {
+      grn_geo_cursor_entry entry;
+      grn_geo_point entry_base;
+      if (!grn_geo_cursor_entry_next(ctx, cursor, &entry)) {
+        cursor->rest = 0;
+        return;
+      }
+      grn_ntog((uint8_t*)(&entry_base), entry.key, sizeof(grn_geo_point));
       if (!(cursor->pat_cursor = pat_cursor =
             grn_table_cursor_open(ctx,
                                   pat,
-                                  base,
-                                  diff_bit,
+                                  &entry_base,
+                                  entry.target_bit + 1,
                                   NULL, 0,
                                   0, -1,
                                   GRN_CURSOR_PREFIX|GRN_CURSOR_SIZE_BY_BIT))) {
@@ -1008,20 +1416,7 @@ grn_geo_cursor_each(grn_ctx *ctx, grn_obj *geo_cursor,
         return;
       }
 #ifdef GEO_DEBUG
-      {
-        switch (direction) {
-        case GRN_GEO_MESH_LATITUDE :
-          mesh_point = base->latitude;
-          break;
-        case GRN_GEO_MESH_LONGITUDE :
-          mesh_point = base->longitude;
-          break;
-        }
-        printf("mesh-point:          %10d\n", mesh_point);
-        inspect_mesh(ctx, base, diff_bit,
-                     (mesh_point - cursor->start_mesh_point) /
-                     distance);
-      }
+      inspect_mesh(ctx, &entry_base, entry.target_bit, 0);
 #endif
     }
 
@@ -1066,19 +1461,6 @@ grn_geo_cursor_each(grn_ctx *ctx, grn_obj *geo_cursor,
     }
     grn_table_cursor_close(ctx, pat_cursor);
     cursor->pat_cursor = pat_cursor = NULL;
-
-    switch (direction) {
-    case GRN_GEO_MESH_LATITUDE :
-      mesh_point = (base->latitude += distance);
-      break;
-    case GRN_GEO_MESH_LONGITUDE :
-      mesh_point = (base->longitude += distance);
-      break;
-    }
-    if (mesh_point > end_mesh_point + distance) {
-      cursor->rest = 0;
-      return;
-    }
   }
 }
 
@@ -1137,13 +1519,11 @@ grn_geo_select_in_rectangle(grn_ctx *ctx, grn_obj *index,
                             grn_obj *res, grn_operator op)
 {
   grn_obj *cursor;
-  grn_posting *posting;
 
   cursor = grn_geo_cursor_open_in_rectangle(ctx, index,
                                             top_left_point, bottom_right_point,
                                             0, -1);
   if (cursor) {
-    grn_geo_cursor_in_rectangle *geo_cursor = (grn_geo_cursor_in_rectangle *)cursor;
     grn_geo_select_in_rectangle_data data;
     data.res = (grn_hash *)res;
     data.op = op;
@@ -1215,6 +1595,7 @@ grn_geo_estimate_in_rectangle(grn_ctx *ctx,
     int select_latitude_distance, select_longitude_distance;
     int total_latitude_distance, total_longitude_distance;
     double select_ratio;
+    double estimated_n_records;
 
     rc = geo_point_get(ctx, data.pat, GRN_CURSOR_ASCENDING, &min);
     if (!rc) {
@@ -1244,7 +1625,8 @@ grn_geo_estimate_in_rectangle(grn_ctx *ctx,
       select_ratio *= ((double)select_longitude_distance /
                        (double)total_longitude_distance);
     }
-    n = (int)(ceil(total_records * select_ratio));
+    estimated_n_records = ceil(total_records * select_ratio);
+    n = (int)estimated_n_records;
   }
 
 exit :
