@@ -54,10 +54,24 @@ extern "C" {
   _longitude = GRN_GEO_INT2RAD(_val->longitude);\
 } while (0)
 
-typedef enum _grn_geo_mesh_direction grn_geo_mesh_direction;
-enum _grn_geo_mesh_direction {
-  GRN_GEO_MESH_LATITUDE,
-  GRN_GEO_MESH_LONGITUDE
+#define GRN_GEO_KEY_MAX_BITS 64
+
+typedef enum _grn_geo_cursor_entry_status_flag grn_geo_cursor_entry_status_flag;
+enum _grn_geo_cursor_entry_status_flag {
+  GRN_GEO_CURSOR_ENTRY_STATUS_NONE            = 0,
+  GRN_GEO_CURSOR_ENTRY_STATUS_TOP_INCLUDED    = 1 << 0,
+  GRN_GEO_CURSOR_ENTRY_STATUS_BOTTOM_INCLUDED = 1 << 1,
+  GRN_GEO_CURSOR_ENTRY_STATUS_LEFT_INCLUDED   = 1 << 2,
+  GRN_GEO_CURSOR_ENTRY_STATUS_RIGHT_INCLUDED  = 1 << 3,
+  GRN_GEO_CURSOR_ENTRY_STATUS_LATITUDE_INNER  = 1 << 4,
+  GRN_GEO_CURSOR_ENTRY_STATUS_LONGITUDE_INNER = 1 << 5
+};
+
+typedef struct _grn_geo_cursor_entry grn_geo_cursor_entry;
+struct _grn_geo_cursor_entry {
+  uint8_t key[sizeof(grn_geo_point)];
+  int target_bit;
+  int status_flags;
 };
 
 typedef struct _grn_geo_cursor_in_rectangle grn_geo_cursor_in_rectangle;
@@ -65,19 +79,18 @@ struct _grn_geo_cursor_in_rectangle {
   grn_db_obj obj;
   grn_obj *pat;
   grn_obj *index;
-  int diff_bit;
-  int start_mesh_point;
-  int end_mesh_point;
-  int distance;
-  grn_geo_mesh_direction direction;
   grn_geo_point top_left;
   grn_geo_point bottom_right;
-  grn_geo_point base;
+  uint8_t top_left_key[sizeof(grn_geo_point)];
+  uint8_t bottom_right_key[sizeof(grn_geo_point)];
   grn_geo_point current;
   grn_table_cursor *pat_cursor;
   grn_ii_cursor *ii_cursor;
   int offset;
   int rest;
+  grn_geo_cursor_entry entries[GRN_GEO_KEY_MAX_BITS];
+  int current_entry;
+  int minimum_reduce_bit;
 };
 
 grn_rc grn_geo_cursor_close(grn_ctx *ctx, grn_obj *geo_cursor);
