@@ -128,7 +128,6 @@ tiny_array_init(grn_ctx *ctx, grn_array *array, const char *path,
     ERR(GRN_INVALID_ARGUMENT, "failed to create array");
     return ctx->rc;
   }
-  GRN_DB_OBJ_SET_TYPE(array, GRN_TABLE_NO_KEY);
   array->obj.header.flags = flags;
   array->ctx = ctx;
   array->value_size = value_size;
@@ -172,7 +171,6 @@ io_array_init(grn_ctx *ctx, grn_array *array, const char *path,
   header->n_entries = 0;
   header->n_garbages = 0;
   header->garbages = GRN_ID_NIL;
-  GRN_DB_OBJ_SET_TYPE(array, GRN_TABLE_NO_KEY);
   array->obj.header.flags = flags;
   array->ctx = ctx;
   array->value_size = value_size;
@@ -203,6 +201,7 @@ grn_array_create(grn_ctx *ctx, const char *path, uint32_t value_size, uint32_t f
 {
   grn_array *array;
   if (ctx && (array = GRN_MALLOC(sizeof(grn_array)))) {
+    GRN_DB_OBJ_SET_TYPE(array, GRN_TABLE_NO_KEY);
     if (_grn_array_create(ctx, array, path, value_size, flags)) {
       return array;
     }
@@ -866,7 +865,6 @@ io_hash_init(grn_hash *ih, grn_ctx *ctx, const char *path, uint32_t key_size,
   header->n_entries = 0;
   header->n_garbages = 0;
   header->tokenizer = 0;
-  GRN_DB_OBJ_SET_TYPE(ih, GRN_TABLE_HASH_KEY);
   ih->obj.header.flags = flags;
   ih->ctx = ctx;
   ih->key_size = key_size;
@@ -906,7 +904,6 @@ tiny_hash_init(grn_hash *ah, grn_ctx *ctx, const char *path, uint32_t key_size,
   if (entry_size != sizeof(uint32_t)) {
     entry_size = ((entry_size + (sizeof(intptr_t)) - 1) & ~((sizeof(intptr_t)) - 1));
   }
-  GRN_DB_OBJ_SET_TYPE(ah, GRN_TABLE_HASH_KEY);
   ah->obj.header.flags = flags;
   ah->ctx = ctx;
   ah->key_size = key_size;
@@ -949,6 +946,7 @@ grn_hash_create(grn_ctx *ctx, const char *path, uint32_t key_size, uint32_t valu
   if (!(hash = GRN_MALLOC(sizeof(grn_hash)))) {
     return NULL;
   }
+  GRN_DB_OBJ_SET_TYPE(hash, GRN_TABLE_HASH_KEY);
   if (!_grn_hash_create(ctx, hash, path, key_size, value_size, flags)) {
     GRN_FREE(hash);
     return NULL;
@@ -1671,8 +1669,8 @@ grn_hash_cursor_open(grn_ctx *ctx, grn_hash *hash,
   } else {
     c->curr_rec += c->dir * offset;
   }
-  c->rest = (limit < 0) ? GRN_ID_MAX : limit;
 exit :
+  c->rest = (limit < 0) ? GRN_ID_MAX : limit;
   return c;
 }
 
@@ -2171,48 +2169,6 @@ grn_hash_check(grn_ctx *ctx, grn_hash *hash)
   GRN_OUTPUT_ARRAY_CLOSE();
 }
 
-/* todo : not completed yet
-grn_rc
-grn_hash_group(grn_ctx *ctx, grn_hash *hash,
-               int n_args, grn_table_group_arg *args)
-{
-  grn_id id;
-  uint32_t n = *hash->n_entries;
-  GRN_ASSERT(n_args && args);
-  {
-    int i;
-    for (i = 0; i < n_args; i++) {
-      ((grn_obj_db *)args[i].result)->range = ((grn_obj_db *)hash)->id;
-    }
-  }
-  for (id = 1; n; id++) {
-    uint32_t es;
-    entry_str *e;
-    uint8_t exist;
-    const uint8_t *ep;
-    grn_table_sort_optarg *arg = &args->sortarg;
-    BITMAP_AT(hash, id, exist);
-    if (!exist) { continue; }
-    n--;
-    ENTRY_AT(hash, id, e, 0);
-    if (!e) { return GRN_END_OF_DATA; }
-    PREPARE_VAL(e, ep, es);
-    {
-      uint32_t *val;
-      grn_id gid = (args->result->header.type == GRN_TABLE_PAT_KEY)
-        ? grn_pat_add(ctx, (grn_pat *)args->result, ep, es, (void **)&val, NULL)
-        : grn_hash_add(ctx, (grn_hash *)args->result, ep, es, (void **)&val, NULL);
-      if (!gid) { return ctx->rc; }
-      if (args->flags & GRN_TABLE_GROUP_CALC_COUNT) {
-        *val += 1;
-        val++;
-      }
-    }
-  }
-  return GRN_SUCCESS;
-}
-*/
-
 /* rhash : grn_hash with subrecs */
 
 #ifdef USE_GRN_INDEX2
