@@ -38,6 +38,57 @@ const char *grn_document_root = NULL;
 
 #define VAR GRN_PROC_GET_VAR_BY_OFFSET
 
+/* bulk must be initialized grn_bulk or grn_msg */
+static int
+grn_bulk_put_from_file(grn_ctx *ctx, grn_obj *bulk, const char *path)
+{
+  /* FIXME: implement more smartly with grn_bulk */
+  int fd, ret = 0;
+  struct stat stat;
+  if ((fd = open(path, O_RDONLY|O_NOFOLLOW)) == -1) {
+    switch (errno) {
+    case EACCES :
+      ERR(GRN_OPERATION_NOT_PERMITTED, "request is not allowed: <%s>", path);
+      break;
+    case ENOENT :
+      ERR(GRN_NO_SUCH_FILE_OR_DIRECTORY, "no such file: <%s>", path);
+      break;
+#ifndef WIN32
+    case ELOOP :
+      ERR(GRN_NO_SUCH_FILE_OR_DIRECTORY,
+          "symbolic link is not allowed: <%s>", path);
+      break;
+#endif /* WIN32 */
+    default :
+      ERR(GRN_UNKNOWN_ERROR, "open() failed(errno: %d): <%s>", errno, path);
+      break;
+    }
+    return 0;
+  }
+  if (fstat(fd, &stat) != -1) {
+    char *buf, *bp;
+    off_t rest = stat.st_size;
+    if ((buf = GRN_MALLOC(rest))) {
+      ssize_t ss;
+      for (bp = buf; rest; rest -= ss, bp += ss) {
+        if ((ss = read(fd, bp, rest)) == -1) { goto exit; }
+      }
+      GRN_TEXT_PUT(ctx, bulk, buf, stat.st_size);
+      ret = 1;
+    }
+    GRN_FREE(buf);
+  } else {
+    ERR(GRN_INVALID_ARGUMENT, "cannot stat file: <%s>", path);
+  }
+exit :
+  close(fd);
+  return ret;
+}
+
+#ifdef stat
+#  undef stat
+#endif /* stat */
+
 /**** query expander ****/
 
 static grn_rc
@@ -202,7 +253,7 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
     drilldown_len + 1 + drilldown_sortby_len + 1 + drilldown_output_columns_len +
     match_escalation_threshold_len + 1 + query_expansion_len + 1 +
     sizeof(grn_content_type) + sizeof(int) * 4;
-  long long int threshold, original_threshold;
+  long long int threshold, original_threshold = 0;
   if (cache_key_size <= GRN_TABLE_MAX_KEY_SIZE) {
     grn_obj *cache;
     char *cp = cache_key;
@@ -1161,53 +1212,6 @@ proc_table_list(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_dat
   return NULL;
 }
 
-/* bulk must be initialized grn_bulk or grn_msg */
-static int
-grn_bulk_put_from_file(grn_ctx *ctx, grn_obj *bulk, const char *path)
-{
-  /* FIXME: implement more smartly with grn_bulk */
-  int fd, ret = 0;
-  struct stat stat;
-  if ((fd = open(path, O_RDONLY|O_NOFOLLOW)) == -1) {
-    switch (errno) {
-    case EACCES :
-      ERR(GRN_OPERATION_NOT_PERMITTED, "request is not allowed: <%s>", path);
-      break;
-    case ENOENT :
-      ERR(GRN_NO_SUCH_FILE_OR_DIRECTORY, "no such file: <%s>", path);
-      break;
-#ifndef WIN32
-    case ELOOP :
-      ERR(GRN_NO_SUCH_FILE_OR_DIRECTORY,
-          "symbolic link is not allowed: <%s>", path);
-      break;
-#endif /* WIN32 */
-    default :
-      ERR(GRN_UNKNOWN_ERROR, "open() failed(errno: %d): <%s>", errno, path);
-      break;
-    }
-    return 0;
-  }
-  if (fstat(fd, &stat) != -1) {
-    char *buf, *bp;
-    off_t rest = stat.st_size;
-    if ((buf = GRN_MALLOC(rest))) {
-      ssize_t ss;
-      for (bp = buf; rest; rest -= ss, bp += ss) {
-        if ((ss = read(fd, bp, rest)) == -1) { goto exit; }
-      }
-      GRN_TEXT_PUT(ctx, bulk, buf, stat.st_size);
-      ret = 1;
-    }
-    GRN_FREE(buf);
-  } else {
-    ERR(GRN_INVALID_ARGUMENT, "cannot stat file: <%s>", path);
-  }
-exit :
-  close(fd);
-  return ret;
-}
-
 static grn_obj *
 proc_missing(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
@@ -1505,7 +1509,7 @@ proc_get_resolve_parameters(grn_ctx *ctx, grn_user_data *user_data, grn_obj **ta
 static grn_obj *
 proc_get(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
-  grn_id id;
+  grn_id id = GRN_ID_NIL;
   grn_obj *table = NULL;
   if (!proc_get_resolve_parameters(ctx, user_data, &table, &id)) {
     grn_obj obj;
@@ -1524,13 +1528,14 @@ proc_get(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 static grn_obj *
 proc_delete(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
-  grn_rc rc;
+  grn_rc rc = GRN_INVALID_ARGUMENT;
   grn_obj *table = grn_ctx_get(ctx,
                                GRN_TEXT_VALUE(VAR(0)),
                                GRN_TEXT_LEN(VAR(0)));
   if (table) {
     if (GRN_TEXT_LEN(VAR(1)) && GRN_TEXT_LEN(VAR(2))) {
       ERR(GRN_INVALID_ARGUMENT, "both id and key are specified");
+      rc = ctx->rc;
     } else if (GRN_TEXT_LEN(VAR(1))) {
       grn_obj *p_key = VAR(1);
       grn_obj key;
@@ -1552,10 +1557,12 @@ proc_delete(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
         rc = grn_table_delete_by_id(ctx, table, id);
       } else {
         ERR(GRN_INVALID_ARGUMENT, "invalid id");
+        rc = ctx->rc;
       }
     }
   } else {
     ERR(GRN_INVALID_ARGUMENT, "unknown table name");
+    rc = ctx->rc;
   }
   GRN_OUTPUT_BOOL(!rc);
   return NULL;
