@@ -170,7 +170,7 @@ static int grntest_jobnum;
 static grn_ctx grntest_ctx[MAX_CON];
 static grn_obj *grntest_owndb[MAX_CON];
 
-grn_obj grntest_starttime, grntest_jobs_start;
+static grn_obj grntest_starttime, grntest_jobs_start;
 
 static int
 grntest_atoi(const char *str, const char *end, const char **rest)
@@ -356,11 +356,11 @@ report_command(grn_ctx *ctx, char *command, char *ret, int task_id,
   if (grntest_outtype == OUT_TSV) {
     fprintf(grntest_log_file, "report\t%d\t%s\t%" GRN_FMT_LLD "\t%" GRN_FMT_LLD "\t%.*s\n",
             task_id, GRN_TEXT_VALUE(&escaped_command), start, end,
-            GRN_TEXT_LEN(&result), GRN_TEXT_VALUE(&result));
+            (int)GRN_TEXT_LEN(&result), GRN_TEXT_VALUE(&result));
   } else {
     fprintf(grntest_log_file, "[%d, \"%s\", %" GRN_FMT_LLD ", %" GRN_FMT_LLD ", %.*s],\n",
             task_id, GRN_TEXT_VALUE(&escaped_command), start, end,
-            GRN_TEXT_LEN(&result), GRN_TEXT_VALUE(&result));
+            (int)GRN_TEXT_LEN(&result), GRN_TEXT_VALUE(&result));
   }
   fflush(grntest_log_file);
   GRN_OBJ_FIN(ctx, &escaped_command);
@@ -955,19 +955,22 @@ worker_sub(grn_ctx *ctx, grn_obj *log, int task_id)
 {
   int i, load_mode, load_count;
   grn_obj end_time;
-  long long int latency, self;
+  long long int total_elapsed_time, job_elapsed_time;
   double sec, qps;
   long long int load_start;
+  struct task *task;
+  struct job *job;
 
-  grntest_task[task_id].max = 0LL;
-  grntest_task[task_id].min = 9223372036854775807LL;
-  grntest_task[task_id].qnum = 0;
+  task = &(grntest_task[task_id]);
+  task->max = 0LL;
+  task->min = 9223372036854775807LL;
+  task->qnum = 0;
 
-  for (i = 0; i < grntest_task[task_id].ntimes; i++) {
-    if (grntest_task[task_id].file != NULL) {
+  for (i = 0; i < task->ntimes; i++) {
+    if (task->file != NULL) {
       FILE *fp;
       grn_obj line;
-      fp = fopen(grntest_task[task_id].file, "r");
+      fp = fopen(task->file, "r");
       if (!fp) {
         fprintf(stderr, "Cannot open %s\n",grntest_task[task_id].file);
         error_exit_in_thread(1);
@@ -995,9 +998,9 @@ worker_sub(grn_ctx *ctx, grn_obj *log, int task_id)
         }
         if (load_mode == 1) {
           if (do_load_command(&grntest_ctx[task_id], GRN_TEXT_VALUE(&line),
-                              grntest_task[task_id].jobtype,
+                              task->jobtype,
                               task_id, &load_start)) {
-            grntest_task[task_id].qnum += load_count;
+            task->qnum += load_count;
             load_mode = 0;
             load_count = 0;
             load_start = 0LL;
@@ -1007,9 +1010,9 @@ worker_sub(grn_ctx *ctx, grn_obj *log, int task_id)
           continue;
         }
         do_command(&grntest_ctx[task_id], GRN_TEXT_VALUE(&line),
-                   grntest_task[task_id].jobtype,
+                   task->jobtype,
                    task_id);
-        grntest_task[task_id].qnum++;
+        task->qnum++;
         GRN_BULK_REWIND(&line);
         if (grntest_sigint) {
           goto exit;
@@ -1020,7 +1023,7 @@ worker_sub(grn_ctx *ctx, grn_obj *log, int task_id)
     } else {
       int i, n_commands;
       grn_obj *commands;
-      commands = grntest_task[task_id].commands;
+      commands = task->commands;
       if (!commands) {
         error_exit_in_thread(1);
       }
@@ -1035,17 +1038,17 @@ worker_sub(grn_ctx *ctx, grn_obj *log, int task_id)
         if (load_mode == 1) {
           if (do_load_command(&grntest_ctx[task_id],
                               GRN_TEXT_VALUE(command),
-                              grntest_task[task_id].jobtype, task_id, &load_start)) {
+                              task->jobtype, task_id, &load_start)) {
             load_mode = 0;
             load_start = 0LL;
-            grntest_task[task_id].qnum++;
+            task->qnum++;
           }
           continue;
         }
         do_command(&grntest_ctx[task_id],
                    GRN_TEXT_VALUE(command),
-                   grntest_task[task_id].jobtype, task_id);
-        grntest_task[task_id].qnum++;
+                   task->jobtype, task_id);
+        task->qnum++;
         if (grntest_sigint) {
           goto exit;
         }
@@ -1056,39 +1059,58 @@ worker_sub(grn_ctx *ctx, grn_obj *log, int task_id)
 exit:
   GRN_TIME_INIT(&end_time, 0);
   GRN_TIME_NOW(&grntest_ctx[task_id], &end_time);
-  latency = GRN_TIME_VALUE(&end_time) - GRN_TIME_VALUE(&grntest_starttime);
-  self = GRN_TIME_VALUE(&end_time) - GRN_TIME_VALUE(&grntest_jobs_start);
+  total_elapsed_time = GRN_TIME_VALUE(&end_time) - GRN_TIME_VALUE(&grntest_starttime);
+  job_elapsed_time = GRN_TIME_VALUE(&end_time) - GRN_TIME_VALUE(&grntest_jobs_start);
 
   CRITICAL_SECTION_ENTER(grntest_cs);
-  if (grntest_job[grntest_task[task_id].job_id].max < grntest_task[task_id].max) {
-    grntest_job[grntest_task[task_id].job_id].max = grntest_task[task_id].max;
+  job = &(grntest_job[task->job_id]);
+  if (job->max < task->max) {
+    job->max = task->max;
   }
-  if (grntest_job[grntest_task[task_id].job_id].min > grntest_task[task_id].min) {
-    grntest_job[grntest_task[task_id].job_id].min = grntest_task[task_id].min;
+  if (job->min > task->min) {
+    job->min = task->min;
   }
 
-  grntest_job[grntest_task[task_id].job_id].qnum += grntest_task[task_id].qnum;
-  grntest_job[grntest_task[task_id].job_id].done++;
-  if (grntest_job[grntest_task[task_id].job_id].done ==
-      grntest_job[grntest_task[task_id].job_id].concurrency) {
+  job->qnum += task->qnum;
+  job->done++;
+  if (job->done == job->concurrency) {
     char tmpbuf[BUF_LEN];
-    sec = self / (double)1000000;
-    qps = (double)grntest_job[grntest_task[task_id].job_id].qnum/ sec;
+    sec = job_elapsed_time / (double)1000000;
+    qps = (double)job->qnum/ sec;
     grntest_jobdone++;
     if (grntest_outtype == OUT_TSV) {
       sprintf(tmpbuf,
-              "job\t%s\t%" GRN_FMT_LLD "\t%" GRN_FMT_LLD "\t%f\t%" GRN_FMT_LLD "\t%" GRN_FMT_LLD "\t%d\n",
-              grntest_job[grntest_task[task_id].job_id].jobname, latency, self, qps,
-              grntest_job[grntest_task[task_id].job_id].min,
-              grntest_job[grntest_task[task_id].job_id].max,
-              grntest_job[grntest_task[task_id].job_id].qnum);
+              "job\t"
+              "%s\t"
+              "%" GRN_FMT_LLD "\t"
+              "%" GRN_FMT_LLD "\t"
+              "%f\t"
+              "%" GRN_FMT_LLD "\t"
+              "%" GRN_FMT_LLD "\t"
+              "%d\n",
+              job->jobname,
+              total_elapsed_time,
+              job_elapsed_time,
+              qps,
+              job->min,
+              job->max,
+              job->qnum);
     } else {
       sprintf(tmpbuf,
-              "{\"job\": \"%s\", \"latency\": %" GRN_FMT_LLD ", \"self\": %" GRN_FMT_LLD ", \"qps\": %f, \"min\": %" GRN_FMT_LLD ", \"max\": %" GRN_FMT_LLD ", \"queries\": %d}",
-              grntest_job[grntest_task[task_id].job_id].jobname, latency, self, qps,
-              grntest_job[grntest_task[task_id].job_id].min,
-              grntest_job[grntest_task[task_id].job_id].max,
-              grntest_job[grntest_task[task_id].job_id].qnum);
+              "{\"job\": \"%s\", "
+              "\"total_elapsed_time\": %" GRN_FMT_LLD ", "
+              "\"job_elapsed_time\": %" GRN_FMT_LLD ", "
+              "\"qps\": %f, "
+              "\"min\": %" GRN_FMT_LLD ", "
+              "\"max\": %" GRN_FMT_LLD ", "
+              "\"queries\": %d}",
+              job->jobname,
+              total_elapsed_time,
+              job_elapsed_time,
+              qps,
+              job->min,
+              job->max,
+              job->qnum);
       if (grntest_jobdone < grntest_jobnum) {
         strcat(tmpbuf, ",");
       }
@@ -1101,9 +1123,9 @@ exit:
       } else {
         if (grntest_detail_on) {
           fseek(grntest_log_file, -2, SEEK_CUR);
-          fprintf(grntest_log_file, "], \n");
+          fprintf(grntest_log_file, "],\n");
         }
-        fprintf(grntest_log_file, "\"summary\" :[");
+        fprintf(grntest_log_file, "\"summary\": [");
         fprintf(grntest_log_file, "%.*s",
                 (int)GRN_TEXT_LEN(log), GRN_TEXT_VALUE(log));
         fprintf(grntest_log_file, "]");
@@ -2043,48 +2065,45 @@ printf("%d:type =%d:file=%s:con=%d:ntimes=%d\n", i, grntest_job[i].jobtype,
 
 /* return num of query */
 static int
-do_script(grn_ctx *ctx, const char *sfile)
+do_script(grn_ctx *ctx, const char *script_file_path)
 {
-  int line = 0;
-  int job_num;
-  int qnum, qnum_total = 0;
-  FILE *fp;
-  char buf[BUF_LEN];
+  int n_lines = 0;
+  int n_jobs;
+  int n_queries, total_n_queries = 0;
+  FILE *script_file;
+  grn_obj line;
 
-  fp = fopen(sfile, "r");
-  if (fp == NULL) {
-    fprintf(stderr, "Cannot open script file:%s\n", sfile);
+  script_file = fopen(script_file_path, "r");
+  if (script_file == NULL) {
+    fprintf(stderr, "Cannot open script file: <%s>\n", script_file_path);
     error_exit(ctx, 1);
   }
-  buf[BUF_LEN-2] = '\0';
-  while (fgets(buf, BUF_LEN, fp) != NULL) {
-    if (grntest_sigint ) {
+
+  GRN_TEXT_INIT(&line, 0);
+  while (grn_text_fgets(ctx, &line, script_file) == GRN_SUCCESS) {
+    if (grntest_sigint) {
       break;
     }
-    line++;
-    if (buf[BUF_LEN-2] != '\0') {
-      fprintf(stderr, "Too long line in script file:%d\n", line);
-      error_exit(ctx, 1);
-    }
+    n_lines++;
     grntest_jobdone = 0;
-    job_num = get_jobs(ctx, buf, line);
-    grntest_jobnum = job_num;
+    n_jobs = get_jobs(ctx, GRN_TEXT_VALUE(&line), n_lines);
+    grntest_jobnum = n_jobs;
 
-    if (job_num > 0) {
+    if (n_jobs > 0) {
       GRN_TIME_INIT(&grntest_jobs_start, 0);
       GRN_TIME_NOW(ctx, &grntest_jobs_start);
       if (grntest_outtype == OUT_TSV) {
-        fprintf(grntest_log_file, "jobs-start\t%s\n", buf);
+        fprintf(grntest_log_file, "jobs-start\t%s\n", GRN_TEXT_VALUE(&line));
       } else {
-        fprintf(grntest_log_file, "{\"jobs\": \"%s\",\n", buf);
+        fprintf(grntest_log_file, "{\"jobs\": \"%s\",\n", GRN_TEXT_VALUE(&line));
       }
-      qnum = do_jobs(ctx, job_num, line);
+      n_queries = do_jobs(ctx, n_jobs, n_lines);
       if (grntest_outtype == OUT_TSV) {
-        fprintf(grntest_log_file, "jobs-end\t%s\n", buf);
+        fprintf(grntest_log_file, "jobs-end\t%s\n", GRN_TEXT_VALUE(&line));
       } else {
         fprintf(grntest_log_file, "},\n");
       }
-      qnum_total = qnum_total + qnum;
+      total_n_queries += n_queries;
 
       grn_obj_close(ctx, &grntest_jobs_start);
     }
@@ -2092,9 +2111,13 @@ do_script(grn_ctx *ctx, const char *sfile)
       fprintf(stderr, "Error:Quit\n");
       break;
     }
+    GRN_BULK_REWIND(&line);
   }
-  fclose(fp);
-  return qnum_total;
+  grn_obj_unlink(ctx, &line);
+
+  fclose(script_file);
+
+  return total_n_queries;
 }
 
 static int
@@ -2670,7 +2693,7 @@ cache_file(grn_ctx *ctx, char **flist, char *file, int fnum)
 }
 
 static int
-sync_datafile(grn_ctx *ctx, const char *sfile)
+sync_datafile(grn_ctx *ctx, const char *script_file_path)
 {
   int line = 0;
   int fnum = 0;
@@ -2679,9 +2702,9 @@ sync_datafile(grn_ctx *ctx, const char *sfile)
   char buf[BUF_LEN];
   char *filelist[BUF_LEN];
 
-  fp = fopen(sfile, "r");
+  fp = fopen(script_file_path, "r");
   if (fp == NULL) {
-    fprintf(stderr, "Cannot open script file:%s\n", sfile);
+    fprintf(stderr, "Cannot open script file: <%s>\n", script_file_path);
     error_exit(ctx, 1);
   }
   buf[BUF_LEN-2] = '\0';
@@ -2746,7 +2769,8 @@ usage(void)
          "  -p, --port <port number>:  server port number (default: %d)\n"
          "  --groonga <groonga_path>:  groonga command path (default: %s)\n"
          "  --protocol <gqtp|http>:    groonga server protocol (default: %s)\n"
-         "  --log-path <path>:         specify log file path\n",
+         "  --log-path <path>:         specify log file path\n"
+         "  --pid-path <path>:         specify file path to store PID file\n",
           DEFAULT_DEST, DEFAULT_PORT,
           groonga_path, groonga_protocol);
   exit(1);
@@ -2822,29 +2846,25 @@ get_token(char *line, char *token, int maxlen, char **next)
 }
 
 /* SET_PORT and SET_HOST */
-static int
-check_script(const char *scrname)
+static grn_bool
+check_script(grn_ctx *ctx, const char *script_file_path)
 {
-  FILE *fp;
-  char tmpbuf[BUF_LEN];
+  FILE *script_file;
+  grn_obj line;
   char token[BUF_LEN];
   char prev[BUF_LEN];
   char *next = NULL;
 
-  fp = fopen(scrname, "r");
-  if (!fp) {
-    fprintf(stderr, "check_script:Cannot open script:%s\n", scrname);
-    exit(1);
+  script_file = fopen(script_file_path, "r");
+  if (!script_file) {
+    fprintf(stderr, "Cannot open script file: <%s>\n", script_file_path);
+    return GRN_FALSE;
   }
 
-  tmpbuf[BUF_LEN-2] = '\0';
-  while (fgets(tmpbuf, BUF_LEN, fp) != NULL) {
-    if (tmpbuf[BUF_LEN-2] != '\0') {
-      fprintf(stderr, "Too long line in script:%s\n", scrname);
-      exit(1);
-    }
-    tmpbuf[strlen(tmpbuf)-1] = '\0';
-    get_token(tmpbuf, token, BUF_LEN, &next);
+  GRN_TEXT_INIT(&line, 0);
+  while (grn_text_fgets(ctx, &line, script_file) == GRN_SUCCESS) {
+    GRN_TEXT_VALUE(&line)[GRN_TEXT_LEN(&line) - 1] = '\0';
+    get_token(GRN_TEXT_VALUE(&line), token, BUF_LEN, &next);
     strcpy(prev, token);
 
     while (next) {
@@ -2859,9 +2879,10 @@ check_script(const char *scrname)
       strcpy(prev, token);
     }
   }
+  grn_obj_unlink(ctx, &line);
 
-  fclose(fp);
-  return 0;
+  fclose(script_file);
+  return GRN_TRUE;
 }
 
 #ifndef WIN32
@@ -2917,11 +2938,13 @@ int
 main(int argc, char **argv)
 {
   int qnum, i, mode = 0;
+  int exit_code = EXIT_SUCCESS;
   grn_ctx context;
   char sysinfo[BUF_LEN];
   char log_path_buffer[BUF_LEN];
   const char *log_path = NULL;
-  const char *portstr=NULL, *hoststr=NULL, *dbname=NULL, *scrname=NULL, *outdir=NULL, *outtype=NULL;
+  const char *pid_path = NULL;
+  const char *portstr = NULL, *hoststr = NULL, *dbname = NULL, *scrname = NULL, *outdir = NULL, *outtype = NULL;
   time_t sec;
 
   static grn_str_getopt_opt opts[] = {
@@ -2938,6 +2961,7 @@ main(int argc, char **argv)
     {'\0', "groonga", NULL, 0, getopt_op_none},
     {'\0', "protocol", NULL, 0, getopt_op_none},
     {'\0', "log-path", NULL, 0, getopt_op_none},
+    {'\0', "pid-path", NULL, 0, getopt_op_none},
     {'\0', NULL, NULL, 0, 0}
   };
 
@@ -2948,6 +2972,7 @@ main(int argc, char **argv)
   opts[10].arg = &groonga_path;
   opts[11].arg = &groonga_protocol;
   opts[12].arg = &log_path;
+  opts[13].arg = &pid_path;
 
   i = grn_str_getopt(argc, argv, opts, &mode);
   if (i < 0) {
@@ -2966,7 +2991,20 @@ main(int argc, char **argv)
   default :
     break;
   }
-  
+
+  if (pid_path) {
+    FILE *pid_file;
+    pid_file = fopen(pid_path, "w");
+    if (pid_file) {
+      fprintf(pid_file, "%d", getpid());
+      fclose(pid_file);
+    } else {
+      fprintf(stderr,
+              "failed to open PID file: <%s>: %s\n",
+              pid_path, strerror(errno));
+    }
+  }
+
   if (i < argc) {
     scrname = argv[i];
   }
@@ -3023,7 +3061,10 @@ main(int argc, char **argv)
   if (grntest_ftp_mode) {
     sync_script(&context, scrname);
   }
-  check_script(scrname);
+  if (!check_script(&context, scrname)) {
+    exit_code = EXIT_FAILURE;
+    goto exit;
+  }
 
   start_local(&context, dbname);
   if (!grntest_remote_mode) {
@@ -3082,6 +3123,10 @@ main(int argc, char **argv)
   fprintf(stderr, "grntest done. logfile=%s\n", log_path);
 
 exit:
+  if (pid_path) {
+    remove(pid_path);
+  }
+
   shutdown_server();
 #ifdef WIN32
   if (!grntest_remote_mode) {
@@ -3117,5 +3162,5 @@ exit:
   grn_obj_close(&grntest_server_context, grn_ctx_db(&grntest_server_context));
   grn_ctx_fin(&grntest_server_context);
   grn_fin();
-  return 0;
+  return exit_code;
 }
