@@ -45,8 +45,6 @@
 #define DEFAULT_PORT 8080
 #define DEFAULT_MAX_THREADS 8
 
-grn_rc grn_ctx_close(grn_ctx *ctx);
-
 #define CONST_STR_LEN(x) x, x ? sizeof(x) - 1 : 0
 
 #define LISTEN_BACKLOG 756
@@ -401,8 +399,6 @@ dispatch(void *arg)
   return NULL;
 }
 
-grn_rc grn_text_ulltoa(grn_ctx *ctx, grn_obj *buf, unsigned long long int i);
-
 static void
 msgpack2json(msgpack_object *o, grn_ctx *ctx, grn_obj *buf)
 {
@@ -697,7 +693,7 @@ usage(FILE *output)
 int
 main(int argc, char **argv)
 {
-  int port_no = DEFAULT_PORT, daemon = 0;
+  int port_no = DEFAULT_PORT;
   const char *max_threads_string = NULL, *port_string = NULL;
   const char *address;
   const char *send_endpoint = NULL, *recv_endpoint = NULL, *log_base_path = NULL;
